import { TextEditor } from './text_editor';
import AddText from './add_text.js';

export class ContentContainer {
  constructor(options) {
    this.pageViewer = options.pageViewer
    this.eventBus = options.eventBus
    this.pageIndex = options.pageIndex
    this.pagePtr = options.pagePtr
    this.messageHandler = options.messageHandler
    this.pageDiv = options.pageDiv
    this.viewport = options.viewport
    this.scale = options.scale

    this._cancelled = false
    this.toolMode = this.pageViewer.toolMode || ''
    this.destroyed = false
    this.frameEditorList = []
    this.tool = this.pageViewer.tool || ''
    this.color = this.pageViewer.color || ''
    this._selectedFrameIndex = -1
    this.isSetFont = false

    this.onHandleTool = this.handleTool.bind(this)
    this.onHandleToolMode = this.handleToolMode.bind(this)
  }

  async init() {
    const { editPagePtr } = await this.messageHandler.sendWithPromise('InitEditPage', {
      pagePtr: this.pagePtr
    })
    this.editPagePtr = editPagePtr

    if (!this.isSetFont) {
      try {
        const response = await fetch('/lib/DroidSansFallbackFull.ttf')
        const blob = await response.blob()
        const file = new File([blob], 'DroidSansFallbackFull.ttf', { type: blob.type })
        await this.messageHandler.sendWithPromise('SetFontCallBackForEditPage', {
          editPagePtr: this.editPagePtr,
          file: file
        })
        this.isSetFont = true
      } catch (error) {
        console.log(error)
      }
    }
  }

  async render () {
    if (this._cancelled) {
      return;
    }
    await this.init()
    
    this.eventBus._on('toolChanged', this.onHandleTool)
    this.eventBus._on('toolModeChanged', this.onHandleToolMode)

    await this.messageHandler.send('BeginEdit', {
      editPagePtr: this.editPagePtr,
      type: 1
    })

    const contentContainer = document.createElement('div')
    contentContainer.className = 'contentContainer'
    if (this.toolMode !== 'editor') {
      contentContainer.style.display = 'none'
    }
    this.contentContainer = contentContainer
    this.pageDiv.append(this.contentContainer)

    let editAreaCount = await this.messageHandler.sendWithPromise('GetEditAreaCount', this.editPagePtr)
    
    for (let i = 0; i < editAreaCount; i++) {
      let editAreaPtr = await this.messageHandler.sendWithPromise('GetEditArea', {
        editPagePtr: this.editPagePtr,
        index: i
      })

      if (editAreaPtr) {
        let isTextPtr = await this.messageHandler.sendWithPromise('IsTextArea', editAreaPtr)
        if (isTextPtr) {
          let frameEditor = new TextEditor({
            eventBus: this.eventBus,
            contentContainer: this,
            container: this.contentContainer,
            pagePtr: this.pagePtr,
            editPagePtr: this.editPagePtr,
            editAreaPtr,
            editAreaIndex: i,
            viewport: this.viewport,
            scale: this.scale,
            messageHandler: this.messageHandler,
            pageViewer: this.pageViewer
          })
          this.frameEditorList.push(frameEditor)
        }
      }
    }

    this.destroyed = false
  }

  show () {
    this.contentContainer.style.display = 'block'
  }

  hide () {
    this.contentContainer.style.display = 'none'

    if (this.textManager) {
      this.textManager.tool = ''
      this.textManager.reset()
    }
  }
  
  cancel () {
    this._cancelled = true
    this.destroy()
  }

  destroy () {
    this.contentContainer?.remove()
    this.contentContainer = null
    // console.log(this.frameEditorList)

    if (this.textManager) {
      this.textManager.tool = ''
      this.textManager.reset()
    }

    this.destroyed = true

    this.eventBus._off('toolChanged', this.onHandleTool)
    this.eventBus._off('toolModeChanged', this.onHandleToolMode)
  }

  handleToolMode (mode) {
    this.toolMode = mode
  }

  handleTool({
    tool,
    color
  }) {
    this.tool = tool
    this.color = color

    if (this.textManager) {
      this.textManager.tool = tool
      this.textManager.color = color
    }

    if (tool === 'addText') {
      if (this.textManager) {
        this.textManager.init()
      } else {
        this.textManager = new AddText({
          tool,
          color,
          container: this.pageDiv,
          viewport: this.viewport,
          scale: this.scale,
          page: this.page,
          eventBus: this.eventBus,
          contentContainer: this
        })
      }
    }
  }

  // 添加编辑区域
  async addTextEditor (data) {
    const { rect, fontData, alignType } = data
    const param = {
      pagePtr: this.pagePtr,
      editPagePtr: this.editPagePtr,
      rect,
      fontData,
      alignType
    }
    const editAreaPtr = await this.messageHandler.sendWithPromise('CreateNewTextArea', param)

    let frameEditor = new TextEditor({
      eventBus: this.eventBus,
      contentContainer: this,
      container: this.contentContainer,
      pagePtr: this.pagePtr,
      editPagePtr: this.editPagePtr,
      editAreaPtr,
      editAreaIndex: this.frameEditorList.length,
      viewport: this.viewport,
      scale: this.scale,
      messageHandler: this.messageHandler,
      newAdd: true,
      pageViewer: this.pageViewer
    })
    this.frameEditorList.push(frameEditor)
  }

  get selectedFrameIndex () {
    return this._selectedFrameIndex
  }

  set selectedFrameIndex (index) {
    this._selectedFrameIndex = index
  }

  removeEditor (index) {
    this.frameEditorList.splice(index, 1)
  }
}
